{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE LambdaCase #-}

module WebSocket.AllChat (broadCastAllChatMessage, MonadAllChat) where

import BroadcastUserData (MonadBroadcast (..))
import ClassyPrelude
import Data.Aeson (encode)
import Data.Time.Clock.POSIX
import Data.UUID (UUID)
import Data.UUID.V4 (nextRandom)
import Types.User (clientToUser)
import Types.WebSocketMessages.WebSocketMessages (AllChatMessageIncoming (..), AllChatMessageOutgoing (AllChatMessageOutgoing))
import WebSocket.MonadWebSocketSession (MonadWebSocketSession (getClient))
import WebSocket.WSReaderTApp

class (Monad m) => MonadAllChat m where
  getMessageId :: m UUID
  getTimestamp :: m POSIXTime

instance MonadAllChat (WSApp WSEnv) where
  getMessageId = liftIO nextRandom
  getTimestamp = liftIO getPOSIXTime

broadCastAllChatMessage :: (MonadBroadcast m, MonadWebSocketSession m, MonadAllChat m) => AllChatMessageIncoming -> m ()
broadCastAllChatMessage AllChatMessageIncoming {content = message} = do
  getClient >>= \case
    Nothing -> return ()
    Just client -> do
      uuid <- getMessageId
      timestamp <- getTimestamp
      let broadCastValue = AllChatMessageOutgoing message (clientToUser client) uuid timestamp
      broadCastToClients $ (decodeUtf8 . toStrict . encode) broadCastValue
